/*
 * ----- WIRING ------
 * Microphone:
 *            -  -  Arduino GND
 *            +  -  Arduino AREF and Arduino 3.3V (use breadboard to connect all the wires together
 *            A0 -  Arduino A0
 *            
 * Neopixel:
 *            G  -  Arduino GND
 *            V  -  Arduino 5V
 *            DI -  Arduino pin 6
 *
 *  Microphone value can be adjusted by rotating potentiometer (blue box on the microphone)
 *  If lighting seems to flickering even when the music is off, increase value in the 
 *  
 */
#include <Adafruit_NeoPixel.h>
#include <math.h>

#ifdef __AVR__
  #include <avr/power.h>
#endif

#define N_PIXELS 24  // Number of pixels in strip
#define MIC_PIN   A0  // Microphone is attached to this analog pin
#define LED_PIN    6// NeoPixel LED strip is connected to this pin


int count = 0;
int brightness = 255;
float volume = 0;
float last = 0;
float maxVolume = 0;
float minVolume = 1024;
float avgVolume = 0;
int currentPixel = 0;
bool musicOn = true;


Adafruit_NeoPixel strip = Adafruit_NeoPixel(N_PIXELS, LED_PIN, NEO_GRB + NEO_KHZ800);


//fullPulse variables
int paletteThreshold = 1500;
int colourCount = 1;

void setup() {
  analogReference(EXTERNAL);
  strip.setBrightness(50);
  Serial.begin(9600);

  // initializing neopixel strip
  strip.begin();
  strip.show(); 

   for(int i = 0 ; i < N_PIXELS; i++){
         strip.setPixelColor(i, strip.Color(255, i*2, 30));
         delay(20);
         strip.show();
       }
  strip.clear();
  
  volume = analogRead(MIC_PIN);
}

void loop() {
  
  last = volume;
  
  volume = analogRead(MIC_PIN);
  
  musicOn = abs(last - volume) > 1;// change this value to lower (to make the microphone more sensitive) or higher(to make microphone less sensitive)
  Serial.println(volume);
  
  if (musicOn){
    brightness = 255;

      avgVolume = (avgVolume + volume)/2.0;

      if (volume > maxVolume) maxVolume = volume;
      if (volume < minVolume) minVolume = volume;


      //Choose one of the lighting schemes by commenting the other ones
      PulseColours();
      //PulseFull();
      //QuaterPulse();
      
   } else if(currentPixel != 0){
     strip.setPixelColor(currentPixel, strip.Color(0,0,0));
     currentPixel = currentPixel - 1;
    }
  else {
     brightness -= 5;
     strip.setBrightness(brightness);
   }
   
  strip.show(); 
  count++; // keeps count to enable reset minVolume and MaxVolume
  if (count > 500){
    minVolume = avgVolume;
    maxVolume = avgVolume;
    count = 0;
    
    }
  

  delay(10);
}

void PulseColours(){
  
  if (musicOn){
    
    int pulseSize = map(volume, minVolume, maxVolume, 0,  N_PIXELS);
    currentPixel = pulseSize;
    for (int i = 0; i < N_PIXELS; i++){
      
      if (i < pulseSize){
        //strip.setPixelColor(i,Wheel(map(i, 0, N_PIXELS-1 ,30, 150)));
        strip.setPixelColor(i,Ocean(map(i,0,strip.numPixels()-1,0,paletteThreshold))); //use this line to change to another colour scheme
                                                                                       //don't forget to comment out the one above.
      } else{
        strip.setPixelColor(i, strip.Color(0,0,0)); 
        }
        
    }
    

    //strip.setBrightness(255.0 * pow(volume / maxVolume, 2));
    strip.show();
    }
  }


void PulseFull(){
  
  if (musicOn){
    
    int pulseSize = map(volume, minVolume, maxVolume, 0,N_PIXELS);
    if (pulseSize > N_PIXELS/2){
      colourCount += (paletteThreshold/70)*pulseSize;
      }
    colourCount += 1;
    
    if (colourCount >= paletteThreshold){
      colourCount = 0;
      }

    uint32_t col = Sulfur(colourCount);
    
    for (int i = 0; i < N_PIXELS; i++){
        strip.setPixelColor(i,col);

    }

    strip.setBrightness(255.0 * pow((volume / maxVolume),2.5));
    strip.show();
    }
  }

//Pulsing quarter lighting. 
void QuaterPulse(){
  if (musicOn){
    int division = 4;
    int equalParts = N_PIXELS/division;
    int pulseSize = map(volume, minVolume, maxVolume, 0, equalParts);
    int pixel = 0;
    
    for (int i = 0; i < equalParts; i++){
      for (int j = 0; j < equalParts; j++){
        pixel = i* equalParts + j;
        if (j < pulseSize){
          strip.setPixelColor(pixel,Ocean(map(pixel,0,strip.numPixels()-1,0,paletteThreshold))); //Choose your color scheme on this line!
        } else {
          strip.setPixelColor(pixel, strip.Color(0,0,0)); 
        }
      }
    
    }
    //strip.setBrightness(255.0 * pow(volume / maxVolume, 2));
    strip.show();
  
  }
}

void Circle(){
  
  if (musicOn){
    
    int pulseSize = map(volume, minVolume, maxVolume, 0,  N_PIXELS);
    currentPixel = pulseSize;
    for (int i = 0; i < N_PIXELS; i++){
      
      if (i < pulseSize){
        //strip.setPixelColor(i,Wheel(map(i, 0, N_PIXELS-1 ,30, 150)));
        strip.setPixelColor(i,Ocean(map(i,0,strip.numPixels()-1,0,paletteThreshold))); //use this line to change to another colour scheme
                                                                                       //don't forget to comment out the one above.
      } else{
        strip.setPixelColor(i, strip.Color(0,0,0)); 
        }
        
    }
    

    //strip.setBrightness(255.0 * pow(volume / maxVolume, 2));
    strip.show();
    }
  }

uint32_t Wheel(byte WheelPos) {
  
  if(WheelPos < 85) {
    return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  } 
  else if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } 
  else {
    WheelPos -= 170;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
}


uint32_t Rainbow(unsigned int i) {
  paletteThreshold = 1529;
  if (i > 1529) return Rainbow(i % 1530);
  if (i > 1274) return strip.Color(255, 0, 255 - (i % 255));   //violet -> red
  if (i > 1019) return strip.Color((i % 255), 0, 255);         //blue -> violet
  if (i > 764) return strip.Color(0, 255 - (i % 255), 255);    //aqua -> blue
  if (i > 509) return strip.Color(0, 255, (i % 255));          //green -> aqua
  if (i > 255) return strip.Color(255 - (i % 255), 255, 0);    //yellow -> green
  return strip.Color(255, i, 0);                               //red -> yellow
}

uint32_t Sunset(unsigned int i) {
  paletteThreshold = 1019;
  if (i > 1019) return Sunset(i % 1020);
  if (i > 764) return strip.Color((i % 255), 0, 255 - (i % 255));          //blue -> red
  if (i > 509) return strip.Color(255 - (i % 255), 0, 255);                //purple -> blue
  if (i > 255) return strip.Color(255, 128 - (i % 255) / 2, (i % 255));    //orange -> purple
  return strip.Color(255, i / 2, 0);                                       //red -> orange
}
uint32_t Ocean(unsigned int i) {
  paletteThreshold = 764;
  if (i > 764) return Ocean(i % 765);
  if (i > 509) return strip.Color(0, i % 255, 255 - (i % 255));  //blue -> green
  if (i > 255) return strip.Color(0, 255 - (i % 255), 255);      //aqua -> blue
  return strip.Color(0, 255, i);                                 //green -> aqua
}

uint32_t PinaColada(unsigned int i) {
  paletteThreshold = 764;
  if (i > 764) return PinaColada(i % 765);
  if (i > 509) return strip.Color(255 - (i % 255) / 2, (i % 255) / 2, (i % 255) / 2);  //red -> half white
  if (i > 255) return strip.Color(255, 255 - (i % 255), 0);                            //yellow -> red
  return strip.Color(128 + (i / 2), 128 + (i / 2), 128 - i / 2);                       //half white -> yellow
}

uint32_t Sulfur(unsigned int i) {
  paletteThreshold = 764;
  if (i > 764) return Sulfur(i % 765);
  if (i > 509) return strip.Color(i % 255, 255, 255 - (i % 255));   //aqua -> yellow
  if (i > 255) return strip.Color(0, 255, i % 255);                 //green -> aqua
  return strip.Color(255 - i, 255, 0);                              //yellow -> green
}

uint32_t NoGreen(unsigned int i) {
  paletteThreshold = 1274;
  if (i > 1274) return NoGreen(i % 1275);
  if (i > 1019) return strip.Color(255, 0, 255 - (i % 255));         //violet -> red
  if (i > 764) return strip.Color((i % 255), 0, 255);                //blue -> violet
  if (i > 509) return strip.Color(0, 255 - (i % 255), 255);          //aqua -> blue
  if (i > 255) return strip.Color(255 - (i % 255), 255, i % 255);    //yellow -> aqua
  return strip.Color(255, i, 0);                                     //red -> yellow
}
